//+------------------------------------------------------------------+
//|                                                           HP.mq4 |
//+------------------------------------------------------------------+
#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Black
#property indicator_color2 Green
#property indicator_color3 Red

//Global constants
#define pi 3.141592653589793238462643383279502884197169399375105820974944592

//Input parameters
extern int nobs    =3000; //Number of bars to smooth
extern int lambda =1;    //Equivalent to SMA period      
extern double shrinkhl=0.0;

//Indicator buffers
double hpm[];
double hph[];
double hpl[];

//Global variables
int pipdig;
//+----------------------------------------------------------------------------------------+
int init()
{
   //lambda=0.0625/MathPow(MathSin(pi/FiltPer),4);
   IndicatorDigits(Digits);
   SetIndexBuffer(0,hpm);
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,3);
   SetIndexLabel(0,"HPmid");
   SetIndexBuffer(1,hph);
   SetIndexStyle(1,DRAW_NONE,STYLE_SOLID,3);
   SetIndexLabel(1,"HPhigh");
   SetIndexBuffer(2,hpl);
   SetIndexStyle(2,DRAW_NONE,STYLE_SOLID,3);
   SetIndexLabel(2,"HPlow");
   if(Digits==2||Digits==3)
      pipdig=2;
      else
      pipdig=4;
   GlobalVariableSet("Nobs", nobs);
}
//+----------------------------------------------------------------------------------------+
int start()
{
   double m[];
   double h[];
   double l[];
   ArrayResize(m,nobs);
   ArrayResize(h,nobs);
   ArrayResize(l,nobs);
   for(int i=0;i<nobs;i++)
   {
      m[i]=(High[i]+Low[i])/2;
      h[i]=High[i]-shrinkhl*1/MathPow(10,pipdig);
      l[i]=Low[i]+shrinkhl*1/MathPow(10,pipdig);
   }
   HPF(nobs,lambda,m,hpm);
   HPF(nobs,lambda,h,hph);
   HPF(nobs,lambda,l,hpl);
}
// Hodrick-Prescott Filter-----------------------------------------------------------------+
void HPF(int nobs, double lambda, double x[], double& y[])
{
   double a[],b[],c[],H1,H2,H3,H4,H5,HH1,HH2,HH3,HH4,HH5,HB,HC,Z;
   ArrayResize(a,nobs);
   ArrayResize(b,nobs);
   ArrayResize(c,nobs);
        
   a[0]=1.0+lambda;
   b[0]=-2.0*lambda;
   c[0]=lambda;
   for(int i=1;i<nobs-2;i++)
   {
      a[i]=6.0*lambda+1.0;
      b[i]=-4.0*lambda;
      c[i]=lambda;
   }
   a[1]=5.0*lambda+1;
   a[nobs-1]=1.0+lambda;
   a[nobs-2]=5.0*lambda+1.0;
   b[nobs-2]=-2.0*lambda;
   b[nobs-1]=0.0;
   c[nobs-2]=0.0;
   c[nobs-1]=0.0;
   
   //Forward
   for(i=0;i<nobs;i++)
   {
      Z=a[i]-H4*H1-HH5*HH2;
      HB=b[i];
      HH1=H1;
      H1=(HB-H4*H2)/Z;
      b[i]=H1;
      HC=c[i];
      HH2=H2;
      H2=HC/Z;
      c[i]=H2;
      a[i]=(x[i]-HH3*HH5-H3*H4)/Z;
      HH3=H3;
      H3=a[i];
      H4=HB-H5*HH1;
      HH5=H5;
      H5=HC;
   }
   
   //Backward 
   H2=0;
   H1=a[nobs-1];
   y[nobs-1]=H1;
   for(i=nobs-2;i>=0;i--)
   {
      y[i]=a[i]-b[i]*H1-c[i]*H2;
      H2=H1;
      H1=y[i];
      //Print(i,"  ",y[i],"  ",a[i],"  ",b[i],"  ",H1,"  ",c[i],"  ",H2);
   }
}

